from distutils.core import setup
setup(
	name='sparkaid',
	packages=['sparkaid'],
	version='0.1-beta',
	license='gpl-3.0',
	description='utils for working with Spark',
	author='Averell Levan',
	author_email='lvhuyen@gmail.com',
	url='https://github.com/lvhuyen/SparkAid',
	keywords=['SPARK', 'DATAFRAME', 'MANIPULATE'],
	install_requires=[

	],
	classifiers=[
		'Development Status :: 4 - Beta',
		'Intended Audience :: Developers',
		'Topic :: Software Development :: Build Tools',
		'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',
		'Programming Language :: Python :: 3',
		'Programming Language :: Python :: 3.4',
		'Programming Language :: Python :: 3.5',
		'Programming Language :: Python :: 3.6',
	],
)